@@ -18,19 +18,21 @@ use std::{
 };
 
 use criterion::{criterion_group, criterion_main, Criterion};
-use event::StateEvent;
+use event::PduEvent;
 use js_int::{int, uint};
 use maplit::{btreemap, hashmap, hashset};
-use ruma_common::MilliSecondsSinceUnixEpoch;
-use ruma_events::{
-    pdu::{EventHash, Pdu, RoomV3Pdu},
-    room::{
-        join_rules::{JoinRule, RoomJoinRulesEventContent},
-        member::{MembershipState, RoomMemberEventContent},
+use ruma_common::{
+    events::{
+        pdu::{EventHash, Pdu, RoomV3Pdu},
+        room::{
+            join_rules::{JoinRule, RoomJoinRulesEventContent},
+            member::{MembershipState, RoomMemberEventContent},
+        },
+        RoomEventType, StateEventType,
     },
-    EventType,
+    room_id, user_id, EventId, MilliSecondsSinceUnixEpoch, OwnedEventId, RoomId, RoomVersionId,
+    UserId,
 };
-use ruma_identifiers::{room_id, user_id, EventId, RoomId, RoomVersionId, UserId};
 use ruma_state_res::{self as state_res, Error, Event, Result, StateMap};
 use serde_json::{
     json,
@@ -103,10 +105,7 @@ fn resolve_deeper_event_set(c: &mut Criterion) {
         ]
         .iter()
         .map(|ev| {
-            (
-                (ev.event_type().to_owned(), ev.state_key().unwrap().to_owned()),
-                ev.event_id().to_owned(),
-            )
+            (ev.event_type().with_state_key(ev.state_key().unwrap()), ev.event_id().to_owned())
         })
         .collect::<StateMap<_>>();
 
@@ -121,10 +120,7 @@ fn resolve_deeper_event_set(c: &mut Criterion) {
         ]
         .iter()
         .map(|ev| {
-            (
-                (ev.event_type().to_owned(), ev.state_key().unwrap().to_owned()),
-                ev.event_id().to_owned(),
-            )
+            (ev.event_type().with_state_key(ev.state_key().unwrap()), ev.event_id().to_owned())
         })
         .collect::<StateMap<_>>();
 
@@ -162,7 +158,7 @@ criterion_main!(benches);
 //  IMPLEMENTATION DETAILS AHEAD
 //
 /////////////////////////////////////////////////////////////////////*/
-struct TestStore<E: Event>(HashMap<Box<EventId>, Arc<E>>);
+struct TestStore<E: Event>(HashMap<OwnedEventId, Arc<E>>);
 
 #[allow(unused)]
 impl<E: Event> TestStore<E> {
@@ -174,7 +170,7 @@ impl<E: Event> TestStore<E> {
     }
 
     /// Returns the events that correspond to the `event_ids` sorted in the same order.
-    fn get_events(&self, room_id: &RoomId, event_ids: &[Box<EventId>]) -> Result<Vec<Arc<E>>> {
+    fn get_events(&self, room_id: &RoomId, event_ids: &[OwnedEventId]) -> Result<Vec<Arc<E>>> {
         let mut events = vec![];
         for id in event_ids {
             events.push(self.get_event(room_id, id)?);
@@ -231,15 +227,15 @@ impl<E: Event> TestStore<E> {
     }
 }
 
-impl TestStore<StateEvent> {
+impl TestStore<PduEvent> {
     #[allow(clippy::type_complexity)]
     fn set_up(
         &mut self,
-    ) -> (StateMap<Box<EventId>>, StateMap<Box<EventId>>, StateMap<Box<EventId>>) {
+    ) -> (StateMap<OwnedEventId>, StateMap<OwnedEventId>, StateMap<OwnedEventId>) {
         let create_event = to_pdu_event::<&EventId>(
             "CREATE",
             alice(),
-            EventType::RoomCreate,
+            RoomEventType::RoomCreate,
             Some(""),
             to_raw_json_value(&json!({ "creator": alice() })).unwrap(),
             &[],
@@ -251,7 +247,7 @@ impl TestStore<StateEvent> {
         let alice_mem = to_pdu_event(
             "IMA",
             alice(),
-            EventType::RoomMember,
+            RoomEventType::RoomMember,
             Some(alice().to_string().as_str()),
             member_content_join(),
             &[cre.clone()],
@@ -262,7 +258,7 @@ impl TestStore<StateEvent> {
         let join_rules = to_pdu_event(
             "IJR",
             alice(),
-            EventType::RoomJoinRules,
+            RoomEventType::RoomJoinRules,
             Some(""),
             to_raw_json_value(&RoomJoinRulesEventContent::new(JoinRule::Public)).unwrap(),
             &[cre.clone(), alice_mem.event_id().to_owned()],
@@ -275,7 +271,7 @@ impl TestStore<StateEvent> {
         let bob_mem = to_pdu_event(
             "IMB",
             bob(),
-            EventType::RoomMember,
+            RoomEventType::RoomMember,
             Some(bob().to_string().as_str()),
             member_content_join(),
             &[cre.clone(), join_rules.event_id().to_owned()],
@@ -286,7 +282,7 @@ impl TestStore<StateEvent> {
         let charlie_mem = to_pdu_event(
             "IMC",
             charlie(),
-            EventType::RoomMember,
+            RoomEventType::RoomMember,
             Some(charlie().to_string().as_str()),
             member_content_join(),
             &[cre, join_rules.event_id().to_owned()],
@@ -297,30 +293,21 @@ impl TestStore<StateEvent> {
         let state_at_bob = [&create_event, &alice_mem, &join_rules, &bob_mem]
             .iter()
             .map(|e| {
-                (
-                    (e.event_type().to_owned(), e.state_key().unwrap().to_owned()),
-                    e.event_id().to_owned(),
-                )
+                (e.event_type().with_state_key(e.state_key().unwrap()), e.event_id().to_owned())
             })
             .collect::<StateMap<_>>();
 
         let state_at_charlie = [&create_event, &alice_mem, &join_rules, &charlie_mem]
             .iter()
             .map(|e| {
-                (
-                    (e.event_type().to_owned(), e.state_key().unwrap().to_owned()),
-                    e.event_id().to_owned(),
-                )
+                (e.event_type().with_state_key(e.state_key().unwrap()), e.event_id().to_owned())
             })
             .collect::<StateMap<_>>();
 
         let expected = [&create_event, &alice_mem, &join_rules, &bob_mem, &charlie_mem]
             .iter()
             .map(|e| {
-                (
-                    (e.event_type().to_owned(), e.state_key().unwrap().to_owned()),
-                    e.event_id().to_owned(),
-                )
+                (e.event_type().with_state_key(e.state_key().unwrap()), e.event_id().to_owned())
             })
             .collect::<StateMap<_>>();
 
@@ -328,27 +315,27 @@ impl TestStore<StateEvent> {
     }
 }
 
-fn event_id(id: &str) -> Box<EventId> {
+fn event_id(id: &str) -> OwnedEventId {
     if id.contains('$') {
         return id.try_into().unwrap();
     }
     format!("${}:foo", id).try_into().unwrap()
 }
 
-fn alice() -> Box<UserId> {
-    user_id!("@alice:foo").to_owned()
+fn alice() -> &'static UserId {
+    user_id!("@alice:foo")
 }
 
-fn bob() -> Box<UserId> {
-    user_id!("@bob:foo").to_owned()
+fn bob() -> &'static UserId {
+    user_id!("@bob:foo")
 }
 
-fn charlie() -> Box<UserId> {
-    user_id!("@charlie:foo").to_owned()
+fn charlie() -> &'static UserId {
+    user_id!("@charlie:foo")
 }
 
-fn ella() -> Box<UserId> {
-    user_id!("@ella:foo").to_owned()
+fn ella() -> &'static UserId {
+    user_id!("@ella:foo")
 }
 
 fn room_id() -> &'static RoomId {
@@ -365,13 +352,13 @@ fn member_content_join() -> Box<RawJsonValue> {
 
 fn to_pdu_event<S>(
     id: &str,
-    sender: Box<UserId>,
-    ev_type: EventType,
+    sender: &UserId,
+    ev_type: RoomEventType,
     state_key: Option<&str>,
     content: Box<RawJsonValue>,
     auth_events: &[S],
     prev_events: &[S],
-) -> Arc<StateEvent>
+) -> Arc<PduEvent>
 where
     S: AsRef<str>,
 {
@@ -383,19 +370,17 @@ where
     let prev_events = prev_events.iter().map(AsRef::as_ref).map(event_id).collect::<Vec<_>>();
 
     let state_key = state_key.map(ToOwned::to_owned);
-    Arc::new(StateEvent {
+    Arc::new(PduEvent {
         event_id: id.try_into().unwrap(),
         rest: Pdu::RoomV3Pdu(RoomV3Pdu {
             room_id: room_id().to_owned(),
-            sender,
+            sender: sender.to_owned(),
             origin_server_ts: MilliSecondsSinceUnixEpoch(ts.try_into().unwrap()),
             state_key,
             kind: ev_type,
             content,
             redacts: None,
             unsigned: btreemap! {},
-            #[cfg(not(feature = "__ci"))]
-            origin: "foo".into(),
             auth_events,
             prev_events,
             depth: uint!(0),
@@ -407,12 +392,12 @@ where
 
 // all graphs start with these input events
 #[allow(non_snake_case)]
-fn INITIAL_EVENTS() -> HashMap<Box<EventId>, Arc<StateEvent>> {
+fn INITIAL_EVENTS() -> HashMap<OwnedEventId, Arc<PduEvent>> {
     vec![
         to_pdu_event::<&EventId>(
             "CREATE",
             alice(),
-            EventType::RoomCreate,
+            RoomEventType::RoomCreate,
             Some(""),
             to_raw_json_value(&json!({ "creator": alice() })).unwrap(),
             &[],
@@ -421,8 +406,8 @@ fn INITIAL_EVENTS() -> HashMap<Box<EventId>, Arc<StateEvent>> {
         to_pdu_event(
             "IMA",
             alice(),
-            EventType::RoomMember,
-            Some(alice().to_string().as_str()),
+            RoomEventType::RoomMember,
+            Some(alice().as_str()),
             member_content_join(),
             &["CREATE"],
             &["CREATE"],
@@ -430,16 +415,16 @@ fn INITIAL_EVENTS() -> HashMap<Box<EventId>, Arc<StateEvent>> {
         to_pdu_event(
             "IPOWER",
             alice(),
-            EventType::RoomPowerLevels,
+            RoomEventType::RoomPowerLevels,
             Some(""),
-            to_raw_json_value(&json!({ "users": { alice().to_string(): 100 } })).unwrap(),
+            to_raw_json_value(&json!({ "users": { alice(): 100 } })).unwrap(),
             &["CREATE", "IMA"],
             &["IMA"],
         ),
         to_pdu_event(
             "IJR",
             alice(),
-            EventType::RoomJoinRules,
+            RoomEventType::RoomJoinRules,
             Some(""),
             to_raw_json_value(&RoomJoinRulesEventContent::new(JoinRule::Public)).unwrap(),
             &["CREATE", "IMA", "IPOWER"],
@@ -448,7 +433,7 @@ fn INITIAL_EVENTS() -> HashMap<Box<EventId>, Arc<StateEvent>> {
         to_pdu_event(
             "IMB",
             bob(),
-            EventType::RoomMember,
+            RoomEventType::RoomMember,
             Some(bob().to_string().as_str()),
             member_content_join(),
             &["CREATE", "IJR", "IPOWER"],
@@ -457,7 +442,7 @@ fn INITIAL_EVENTS() -> HashMap<Box<EventId>, Arc<StateEvent>> {
         to_pdu_event(
             "IMC",
             charlie(),
-            EventType::RoomMember,
+            RoomEventType::RoomMember,
             Some(charlie().to_string().as_str()),
             member_content_join(),
             &["CREATE", "IJR", "IPOWER"],
@@ -466,7 +451,7 @@ fn INITIAL_EVENTS() -> HashMap<Box<EventId>, Arc<StateEvent>> {
         to_pdu_event::<&EventId>(
             "START",
             charlie(),
-            EventType::RoomTopic,
+            RoomEventType::RoomTopic,
             Some(""),
             to_raw_json_value(&json!({})).unwrap(),
             &[],
@@ -475,7 +460,7 @@ fn INITIAL_EVENTS() -> HashMap<Box<EventId>, Arc<StateEvent>> {
         to_pdu_event::<&EventId>(
             "END",
             charlie(),
-            EventType::RoomTopic,
+            RoomEventType::RoomTopic,
             Some(""),
             to_raw_json_value(&json!({})).unwrap(),
             &[],
@@ -489,12 +474,12 @@ fn INITIAL_EVENTS() -> HashMap<Box<EventId>, Arc<StateEvent>> {
 
 // all graphs start with these input events
 #[allow(non_snake_case)]
-fn BAN_STATE_SET() -> HashMap<Box<EventId>, Arc<StateEvent>> {
+fn BAN_STATE_SET() -> HashMap<OwnedEventId, Arc<PduEvent>> {
     vec![
         to_pdu_event(
             "PA",
             alice(),
-            EventType::RoomPowerLevels,
+            RoomEventType::RoomPowerLevels,
             Some(""),
             to_raw_json_value(&json!({ "users": { alice(): 100, bob(): 50 } })).unwrap(),
             &["CREATE", "IMA", "IPOWER"], // auth_events
@@ -503,7 +488,7 @@ fn BAN_STATE_SET() -> HashMap<Box<EventId>, Arc<StateEvent>> {
         to_pdu_event(
             "PB",
             alice(),
-            EventType::RoomPowerLevels,
+            RoomEventType::RoomPowerLevels,
             Some(""),
             to_raw_json_value(&json!({ "users": { alice(): 100, bob(): 50 } })).unwrap(),
             &["CREATE", "IMA", "IPOWER"],
@@ -512,7 +497,7 @@ fn BAN_STATE_SET() -> HashMap<Box<EventId>, Arc<StateEvent>> {
         to_pdu_event(
             "MB",
             alice(),
-            EventType::RoomMember,
+            RoomEventType::RoomMember,
             Some(ella().as_str()),
             member_content_ban(),
             &["CREATE", "IMA", "PB"],
@@ -521,7 +506,7 @@ fn BAN_STATE_SET() -> HashMap<Box<EventId>, Arc<StateEvent>> {
         to_pdu_event(
             "IME",
             ella(),
-            EventType::RoomMember,
+            RoomEventType::RoomMember,
             Some(ella().as_str()),
             member_content_join(),
             &["CREATE", "IJR", "PA"],
@@ -533,16 +518,28 @@ fn BAN_STATE_SET() -> HashMap<Box<EventId>, Arc<StateEvent>> {
     .collect()
 }
 
+/// Convenience trait for adding event type plus state key to state maps.
+trait EventTypeExt {
+    fn with_state_key(self, state_key: impl Into<String>) -> (StateEventType, String);
+}
+
+impl EventTypeExt for &RoomEventType {
+    fn with_state_key(self, state_key: impl Into<String>) -> (StateEventType, String) {
+        (self.to_string().into(), state_key.into())
+    }
+}
+
 mod event {
-    use ruma_common::MilliSecondsSinceUnixEpoch;
-    use ruma_events::{pdu::Pdu, EventType};
-    use ruma_identifiers::{EventId, RoomId, UserId};
+    use ruma_common::{
+        events::{pdu::Pdu, RoomEventType},
+        MilliSecondsSinceUnixEpoch, OwnedEventId, RoomId, UserId,
+    };
     use ruma_state_res::Event;
     use serde::{Deserialize, Serialize};
     use serde_json::value::RawValue as RawJsonValue;
 
-    impl Event for StateEvent {
-        type Id = Box<EventId>;
+    impl Event for PduEvent {
+        type Id = OwnedEventId;
 
         fn event_id(&self) -> &Self::Id {
             &self.event_id
@@ -566,7 +563,7 @@ mod event {
             }
         }
 
-        fn event_type(&self) -> &EventType {
+        fn event_type(&self) -> &RoomEventType {
             match &self.rest {
                 Pdu::RoomV1Pdu(ev) => &ev.kind,
                 Pdu::RoomV3Pdu(ev) => &ev.kind,
@@ -631,8 +628,8 @@ mod event {
     }
 
     #[derive(Clone, Debug, Deserialize, Serialize)]
-    pub struct StateEvent {
-        pub event_id: Box<EventId>,
+    pub struct PduEvent {
+        pub event_id: OwnedEventId,
         #[serde(flatten)]
         pub rest: Pdu,
     }
