@@ -1,4 +1,4 @@
-use ruma_identifiers::RoomVersionId;
+use ruma_common::RoomVersionId;
 
 use crate::{Error, Result};
 
@@ -64,7 +64,7 @@ pub struct RoomVersion {
     pub allow_knocking: bool,
     /// Adds support for the restricted join rule.
     ///
-    /// See: <https://github.com/matrix-org/matrix-doc/pull/3289> for more information.
+    /// See: [MSC3289](https://github.com/matrix-org/matrix-spec-proposals/pull/3289) for more information.
     pub restricted_join_rules: bool,
 }
 
@@ -98,12 +98,7 @@ impl RoomVersion {
         ..Self::V5
     };
 
-    pub const V7: Self = Self {
-        // FIXME: once room version 7 is stabilized move this to version 8
-        disposition: RoomDisposition::Unstable,
-        allow_knocking: true,
-        ..Self::V6
-    };
+    pub const V7: Self = Self { allow_knocking: true, ..Self::V6 };
 
     pub const V8: Self = Self { restricted_join_rules: true, ..Self::V7 };
 
