@@ -9,32 +9,35 @@ use std::{
 };
 
 use js_int::{int, uint};
-use ruma_common::MilliSecondsSinceUnixEpoch;
-use ruma_events::{
-    pdu::{EventHash, Pdu, RoomV3Pdu},
-    room::{
-        join_rules::{JoinRule, RoomJoinRulesEventContent},
-        member::{MembershipState, RoomMemberEventContent},
+use ruma_common::{
+    event_id,
+    events::{
+        pdu::{EventHash, Pdu, RoomV3Pdu},
+        room::{
+            join_rules::{JoinRule, RoomJoinRulesEventContent},
+            member::{MembershipState, RoomMemberEventContent},
+        },
+        RoomEventType,
     },
-    EventType,
+    room_id, user_id, EventId, MilliSecondsSinceUnixEpoch, OwnedEventId, RoomId, RoomVersionId,
+    UserId,
 };
-use ruma_identifiers::{event_id, room_id, user_id, EventId, RoomId, RoomVersionId, UserId};
 use serde_json::{
     json,
     value::{to_raw_value as to_raw_json_value, RawValue as RawJsonValue},
 };
 use tracing::info;
 
-use crate::{auth_types_for_event, Error, Event, Result, StateMap};
+use crate::{auth_types_for_event, Error, Event, EventTypeExt, Result, StateMap};
 
-pub use event::StateEvent;
+pub use event::PduEvent;
 
 static SERVER_TIMESTAMP: AtomicU64 = AtomicU64::new(0);
 
 pub fn do_check(
-    events: &[Arc<StateEvent>],
-    edges: Vec<Vec<Box<EventId>>>,
-    expected_state_ids: Vec<Box<EventId>>,
+    events: &[Arc<PduEvent>],
+    edges: Vec<Vec<OwnedEventId>>,
+    expected_state_ids: Vec<OwnedEventId>,
 ) {
     // To activate logging use `RUST_LOG=debug cargo t`
 
@@ -50,7 +53,7 @@ pub fn do_check(
 
     // This will be lexi_topo_sorted for resolution
     let mut graph = HashMap::new();
-    // This is the same as in `resolve` event_id -> StateEvent
+    // This is the same as in `resolve` event_id -> OriginalStateEvent
     let mut fake_event_map = HashMap::new();
 
     // Create the DB of events that led up to this point
@@ -74,10 +77,10 @@ pub fn do_check(
         }
     }
 
-    // event_id -> StateEvent
-    let mut event_map: HashMap<Box<EventId>, Arc<StateEvent>> = HashMap::new();
-    // event_id -> StateMap<Box<EventId>>
-    let mut state_at_event: HashMap<Box<EventId>, StateMap<Box<EventId>>> = HashMap::new();
+    // event_id -> PduEvent
+    let mut event_map: HashMap<OwnedEventId, Arc<PduEvent>> = HashMap::new();
+    // event_id -> StateMap<OwnedEventId>
+    let mut state_at_event: HashMap<OwnedEventId, StateMap<OwnedEventId>> = HashMap::new();
 
     // Resolve the current state and add it to the state_at_event map then continue
     // on in "time"
@@ -91,7 +94,7 @@ pub fn do_check(
 
         let prev_events = graph.get(&node).unwrap();
 
-        let state_before: StateMap<Box<EventId>> = if prev_events.is_empty() {
+        let state_before: StateMap<OwnedEventId> = if prev_events.is_empty() {
             HashMap::new()
         } else if prev_events.len() == 1 {
             state_at_event.get(prev_events.iter().next().unwrap()).unwrap().clone()
@@ -128,9 +131,9 @@ pub fn do_check(
 
         let mut state_after = state_before.clone();
 
-        let ty = fake_event.event_type().to_owned();
-        let key = fake_event.state_key().unwrap().to_owned();
-        state_after.insert((ty, key), event_id.to_owned());
+        let ty = fake_event.event_type();
+        let key = fake_event.state_key().unwrap();
+        state_after.insert(ty.with_state_key(key), event_id.to_owned());
 
         let auth_types = auth_types_for_event(
             fake_event.event_type(),
@@ -153,7 +156,7 @@ pub fn do_check(
         let ev_id = e.event_id();
         let event = to_pdu_event(
             e.event_id().as_str(),
-            e.sender().to_owned(),
+            e.sender(),
             e.event_type().clone(),
             e.state_key(),
             e.content().to_owned(),
@@ -179,7 +182,7 @@ pub fn do_check(
             )
         });
 
-        let key = (ev.event_type().to_owned(), ev.state_key().unwrap().to_owned());
+        let key = ev.event_type().with_state_key(ev.state_key().unwrap());
 
         expected_state.insert(key, node);
     }
@@ -196,16 +199,16 @@ pub fn do_check(
                 // Filter out the dummy messages events.
                 // These act as points in time where there should be a known state to
                 // test against.
-                && **k != (EventType::RoomMessage, "dummy".to_owned())
+                && **k != ("m.room.message".into(), "dummy".to_owned())
         })
         .map(|(k, v)| (k.clone(), v.clone()))
-        .collect::<StateMap<Box<EventId>>>();
+        .collect::<StateMap<OwnedEventId>>();
 
     assert_eq!(expected_state, end_state);
 }
 
 #[allow(clippy::exhaustive_structs)]
-pub struct TestStore<E: Event>(pub HashMap<Box<EventId>, Arc<E>>);
+pub struct TestStore<E: Event>(pub HashMap<OwnedEventId, Arc<E>>);
 
 impl<E: Event> TestStore<E> {
     pub fn get_event(&self, _: &RoomId, event_id: &EventId) -> Result<Arc<E>> {
@@ -243,14 +246,14 @@ impl<E: Event> TestStore<E> {
 
 // A StateStore implementation for testing
 #[allow(clippy::type_complexity)]
-impl TestStore<StateEvent> {
+impl TestStore<PduEvent> {
     pub fn set_up(
         &mut self,
-    ) -> (StateMap<Box<EventId>>, StateMap<Box<EventId>>, StateMap<Box<EventId>>) {
+    ) -> (StateMap<OwnedEventId>, StateMap<OwnedEventId>, StateMap<OwnedEventId>) {
         let create_event = to_pdu_event::<&EventId>(
             "CREATE",
             alice(),
-            EventType::RoomCreate,
+            RoomEventType::RoomCreate,
             Some(""),
             to_raw_json_value(&json!({ "creator": alice() })).unwrap(),
             &[],
@@ -262,8 +265,8 @@ impl TestStore<StateEvent> {
         let alice_mem = to_pdu_event(
             "IMA",
             alice(),
-            EventType::RoomMember,
-            Some(alice().to_string().as_str()),
+            RoomEventType::RoomMember,
+            Some(alice().as_str()),
             member_content_join(),
             &[cre.clone()],
             &[cre.clone()],
@@ -273,7 +276,7 @@ impl TestStore<StateEvent> {
         let join_rules = to_pdu_event(
             "IJR",
             alice(),
-            EventType::RoomJoinRules,
+            RoomEventType::RoomJoinRules,
             Some(""),
             to_raw_json_value(&RoomJoinRulesEventContent::new(JoinRule::Public)).unwrap(),
             &[cre.clone(), alice_mem.event_id().to_owned()],
@@ -286,8 +289,8 @@ impl TestStore<StateEvent> {
         let bob_mem = to_pdu_event(
             "IMB",
             bob(),
-            EventType::RoomMember,
-            Some(bob().to_string().as_str()),
+            RoomEventType::RoomMember,
+            Some(bob().as_str()),
             member_content_join(),
             &[cre.clone(), join_rules.event_id().to_owned()],
             &[join_rules.event_id().to_owned()],
@@ -297,8 +300,8 @@ impl TestStore<StateEvent> {
         let charlie_mem = to_pdu_event(
             "IMC",
             charlie(),
-            EventType::RoomMember,
-            Some(charlie().to_string().as_str()),
+            RoomEventType::RoomMember,
+            Some(charlie().as_str()),
             member_content_join(),
             &[cre, join_rules.event_id().to_owned()],
             &[join_rules.event_id().to_owned()],
@@ -308,30 +311,21 @@ impl TestStore<StateEvent> {
         let state_at_bob = [&create_event, &alice_mem, &join_rules, &bob_mem]
             .iter()
             .map(|e| {
-                (
-                    (e.event_type().to_owned(), e.state_key().unwrap().to_owned()),
-                    e.event_id().to_owned(),
-                )
+                (e.event_type().with_state_key(e.state_key().unwrap()), e.event_id().to_owned())
             })
             .collect::<StateMap<_>>();
 
         let state_at_charlie = [&create_event, &alice_mem, &join_rules, &charlie_mem]
             .iter()
             .map(|e| {
-                (
-                    (e.event_type().to_owned(), e.state_key().unwrap().to_owned()),
-                    e.event_id().to_owned(),
-                )
+                (e.event_type().with_state_key(e.state_key().unwrap()), e.event_id().to_owned())
             })
             .collect::<StateMap<_>>();
 
         let expected = [&create_event, &alice_mem, &join_rules, &bob_mem, &charlie_mem]
             .iter()
             .map(|e| {
-                (
-                    (e.event_type().to_owned(), e.state_key().unwrap().to_owned()),
-                    e.event_id().to_owned(),
-                )
+                (e.event_type().with_state_key(e.state_key().unwrap()), e.event_id().to_owned())
             })
             .collect::<StateMap<_>>();
 
@@ -339,7 +333,7 @@ impl TestStore<StateEvent> {
     }
 }
 
-pub fn event_id(id: &str) -> Box<EventId> {
+pub fn event_id(id: &str) -> OwnedEventId {
     if id.contains('$') {
         return id.try_into().unwrap();
     }
@@ -347,24 +341,24 @@ pub fn event_id(id: &str) -> Box<EventId> {
     format!("${}:foo", id).try_into().unwrap()
 }
 
-pub fn alice() -> Box<UserId> {
-    user_id!("@alice:foo").to_owned()
+pub fn alice() -> &'static UserId {
+    user_id!("@alice:foo")
 }
 
-pub fn bob() -> Box<UserId> {
-    user_id!("@bob:foo").to_owned()
+pub fn bob() -> &'static UserId {
+    user_id!("@bob:foo")
 }
 
-pub fn charlie() -> Box<UserId> {
-    user_id!("@charlie:foo").to_owned()
+pub fn charlie() -> &'static UserId {
+    user_id!("@charlie:foo")
 }
 
-pub fn ella() -> Box<UserId> {
-    user_id!("@ella:foo").to_owned()
+pub fn ella() -> &'static UserId {
+    user_id!("@ella:foo")
 }
 
-pub fn zara() -> Box<UserId> {
-    user_id!("@zara:foo").to_owned()
+pub fn zara() -> &'static UserId {
+    user_id!("@zara:foo")
 }
 
 pub fn room_id() -> &'static RoomId {
@@ -381,28 +375,26 @@ pub fn member_content_join() -> Box<RawJsonValue> {
 
 pub fn to_init_pdu_event(
     id: &str,
-    sender: Box<UserId>,
-    ev_type: EventType,
+    sender: &UserId,
+    ev_type: RoomEventType,
     state_key: Option<&str>,
     content: Box<RawJsonValue>,
-) -> Arc<StateEvent> {
+) -> Arc<PduEvent> {
     let ts = SERVER_TIMESTAMP.fetch_add(1, SeqCst);
     let id = if id.contains('$') { id.to_owned() } else { format!("${}:foo", id) };
 
     let state_key = state_key.map(ToOwned::to_owned);
-    Arc::new(StateEvent {
+    Arc::new(PduEvent {
         event_id: id.try_into().unwrap(),
         rest: Pdu::RoomV3Pdu(RoomV3Pdu {
             room_id: room_id().to_owned(),
-            sender,
+            sender: sender.to_owned(),
             origin_server_ts: MilliSecondsSinceUnixEpoch(ts.try_into().unwrap()),
             state_key,
             kind: ev_type,
             content,
             redacts: None,
             unsigned: BTreeMap::new(),
-            #[cfg(not(feature = "__ci"))]
-            origin: "foo".into(),
             auth_events: vec![],
             prev_events: vec![],
             depth: uint!(0),
@@ -414,13 +406,13 @@ pub fn to_init_pdu_event(
 
 pub fn to_pdu_event<S>(
     id: &str,
-    sender: Box<UserId>,
-    ev_type: EventType,
+    sender: &UserId,
+    ev_type: RoomEventType,
     state_key: Option<&str>,
     content: Box<RawJsonValue>,
     auth_events: &[S],
     prev_events: &[S],
-) -> Arc<StateEvent>
+) -> Arc<PduEvent>
 where
     S: AsRef<str>,
 {
@@ -430,19 +422,17 @@ where
     let prev_events = prev_events.iter().map(AsRef::as_ref).map(event_id).collect::<Vec<_>>();
 
     let state_key = state_key.map(ToOwned::to_owned);
-    Arc::new(StateEvent {
+    Arc::new(PduEvent {
         event_id: id.try_into().unwrap(),
         rest: Pdu::RoomV3Pdu(RoomV3Pdu {
             room_id: room_id().to_owned(),
-            sender,
+            sender: sender.to_owned(),
             origin_server_ts: MilliSecondsSinceUnixEpoch(ts.try_into().unwrap()),
             state_key,
             kind: ev_type,
             content,
             redacts: None,
             unsigned: BTreeMap::new(),
-            #[cfg(not(feature = "__ci"))]
-            origin: "foo".into(),
             auth_events,
             prev_events,
             depth: uint!(0),
@@ -454,12 +444,12 @@ where
 
 // all graphs start with these input events
 #[allow(non_snake_case)]
-pub fn INITIAL_EVENTS() -> HashMap<Box<EventId>, Arc<StateEvent>> {
+pub fn INITIAL_EVENTS() -> HashMap<OwnedEventId, Arc<PduEvent>> {
     vec![
         to_pdu_event::<&EventId>(
             "CREATE",
             alice(),
-            EventType::RoomCreate,
+            RoomEventType::RoomCreate,
             Some(""),
             to_raw_json_value(&json!({ "creator": alice() })).unwrap(),
             &[],
@@ -468,8 +458,8 @@ pub fn INITIAL_EVENTS() -> HashMap<Box<EventId>, Arc<StateEvent>> {
         to_pdu_event(
             "IMA",
             alice(),
-            EventType::RoomMember,
-            Some(alice().to_string().as_str()),
+            RoomEventType::RoomMember,
+            Some(alice().as_str()),
             member_content_join(),
             &["CREATE"],
             &["CREATE"],
@@ -477,16 +467,16 @@ pub fn INITIAL_EVENTS() -> HashMap<Box<EventId>, Arc<StateEvent>> {
         to_pdu_event(
             "IPOWER",
             alice(),
-            EventType::RoomPowerLevels,
+            RoomEventType::RoomPowerLevels,
             Some(""),
-            to_raw_json_value(&json!({ "users": { alice().to_string(): 100 } })).unwrap(),
+            to_raw_json_value(&json!({ "users": { alice(): 100 } })).unwrap(),
             &["CREATE", "IMA"],
             &["IMA"],
         ),
         to_pdu_event(
             "IJR",
             alice(),
-            EventType::RoomJoinRules,
+            RoomEventType::RoomJoinRules,
             Some(""),
             to_raw_json_value(&RoomJoinRulesEventContent::new(JoinRule::Public)).unwrap(),
             &["CREATE", "IMA", "IPOWER"],
@@ -495,8 +485,8 @@ pub fn INITIAL_EVENTS() -> HashMap<Box<EventId>, Arc<StateEvent>> {
         to_pdu_event(
             "IMB",
             bob(),
-            EventType::RoomMember,
-            Some(bob().to_string().as_str()),
+            RoomEventType::RoomMember,
+            Some(bob().as_str()),
             member_content_join(),
             &["CREATE", "IJR", "IPOWER"],
             &["IJR"],
@@ -504,8 +494,8 @@ pub fn INITIAL_EVENTS() -> HashMap<Box<EventId>, Arc<StateEvent>> {
         to_pdu_event(
             "IMC",
             charlie(),
-            EventType::RoomMember,
-            Some(charlie().to_string().as_str()),
+            RoomEventType::RoomMember,
+            Some(charlie().as_str()),
             member_content_join(),
             &["CREATE", "IJR", "IPOWER"],
             &["IMB"],
@@ -513,7 +503,7 @@ pub fn INITIAL_EVENTS() -> HashMap<Box<EventId>, Arc<StateEvent>> {
         to_pdu_event::<&EventId>(
             "START",
             charlie(),
-            EventType::RoomMessage,
+            RoomEventType::RoomMessage,
             Some("dummy"),
             to_raw_json_value(&json!({})).unwrap(),
             &[],
@@ -522,7 +512,7 @@ pub fn INITIAL_EVENTS() -> HashMap<Box<EventId>, Arc<StateEvent>> {
         to_pdu_event::<&EventId>(
             "END",
             charlie(),
-            EventType::RoomMessage,
+            RoomEventType::RoomMessage,
             Some("dummy"),
             to_raw_json_value(&json!({})).unwrap(),
             &[],
@@ -534,8 +524,25 @@ pub fn INITIAL_EVENTS() -> HashMap<Box<EventId>, Arc<StateEvent>> {
     .collect()
 }
 
+// all graphs start with these input events
 #[allow(non_snake_case)]
-pub fn INITIAL_EDGES() -> Vec<Box<EventId>> {
+pub fn INITIAL_EVENTS_CREATE_ROOM() -> HashMap<OwnedEventId, Arc<PduEvent>> {
+    vec![to_pdu_event::<&EventId>(
+        "CREATE",
+        alice(),
+        RoomEventType::RoomCreate,
+        Some(""),
+        to_raw_json_value(&json!({ "creator": alice() })).unwrap(),
+        &[],
+        &[],
+    )]
+    .into_iter()
+    .map(|ev| (ev.event_id().to_owned(), ev))
+    .collect()
+}
+
+#[allow(non_snake_case)]
+pub fn INITIAL_EDGES() -> Vec<OwnedEventId> {
     vec!["START", "IMC", "IMB", "IJR", "IPOWER", "IMA", "CREATE"]
         .into_iter()
         .map(event_id)
@@ -543,15 +550,17 @@ pub fn INITIAL_EDGES() -> Vec<Box<EventId>> {
 }
 
 pub mod event {
-    use ruma_events::{exports::ruma_common::MilliSecondsSinceUnixEpoch, pdu::Pdu, EventType};
-    use ruma_identifiers::{EventId, RoomId, UserId};
+    use ruma_common::{
+        events::{pdu::Pdu, RoomEventType},
+        MilliSecondsSinceUnixEpoch, OwnedEventId, RoomId, UserId,
+    };
     use serde::{Deserialize, Serialize};
     use serde_json::value::RawValue as RawJsonValue;
 
     use crate::Event;
 
-    impl Event for StateEvent {
-        type Id = Box<EventId>;
+    impl Event for PduEvent {
+        type Id = OwnedEventId;
 
         fn event_id(&self) -> &Self::Id {
             &self.event_id
@@ -575,7 +584,7 @@ pub mod event {
             }
         }
 
-        fn event_type(&self) -> &EventType {
+        fn event_type(&self) -> &RoomEventType {
             match &self.rest {
                 Pdu::RoomV1Pdu(ev) => &ev.kind,
                 Pdu::RoomV3Pdu(ev) => &ev.kind,
@@ -641,20 +650,9 @@ pub mod event {
 
     #[derive(Clone, Debug, Deserialize, Serialize)]
     #[allow(clippy::exhaustive_structs)]
-    pub struct StateEvent {
-        pub event_id: Box<EventId>,
+    pub struct PduEvent {
+        pub event_id: OwnedEventId,
         #[serde(flatten)]
         pub rest: Pdu,
     }
-
-    //impl StateEvent {
-    //    pub fn state_key(&self) -> &str {
-    //        match &self.rest {
-    //            Pdu::RoomV1Pdu(ev) => ev.state_key.as_ref().unwrap(),
-    //            Pdu::RoomV3Pdu(ev) => ev.state_key.as_ref().unwrap(),
-    //            #[cfg(not(feature = "unstable-exhaustive-types"))]
-    //            _ => unreachable!("new PDU version"),
-    //        }
-    //    }
-    //}
 }
